/*
 * FreeRTOS Kernel V10.2.1
 * Copyright (C) 2019 Amazon.com, Inc. or its affiliates.  All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 * the Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * http://www.FreeRTOS.org
 * http://aws.amazon.com/freertos
 *
 * 1 tab == 4 spaces!
 */


#include "FreeRTOSConfig.h"
#include "stdint.h"

/* Variables used by scheduler */
typedef void TCB_t; /* don't know or need to know its type. */
extern volatile TCB_t * volatile pxCurrentTCB;
extern volatile uint16_t usCriticalNesting;

static void vPortSAVE_CONTEXT_C(void);
static void vPortRESTORE_CONTEXT_C(void);

/*
 * vSAVE_CONTEXT_C
 * Saves the context of the general purpose registers, CS and ES registers,
 * the usCriticalNesting Value and the Stack Pointer of the active Task
 * onto the task stack
 */
#pragma inline_asm vPortSAVE_CONTEXT_C
static void vPortSAVE_CONTEXT_C(void)
{
#if !defined(__CDT_PARSER__)
;	It is assumed that the general purpose registers, CS and ES registers
;	and the usCriticalNesting Value have been saved by such as the following
;	code before this inline asm funcsion.
;	PUSH      AX                    ; Save the general purpose registers.
;	PUSH      BC
;	PUSH      DE
;	PUSH      HL
;	MOV       A, ES                 ; Save ES register.
;	MOV       X, A
;	MOV       A, CS                 ; Save CS register.
;	PUSH      AX

;	MOVW      AX, !_usCriticalNesting   ; Save the usCriticalNesting value.
;	PUSH      AX
; or
;	PUSH      HL
;	MOVW      AX, !_usCriticalNesting   ; Save the usCriticalNesting value.
;	MOVW      [SP], AX

	MOVW      AX, !_pxCurrentTCB    ; Save the Stack pointer.
	MOVW      HL, AX
	MOVW      AX, SP
	MOVW      [HL], AX
;	/* Switch stack pointers. */ ; The GNURL78 port has this. Why? (NoMaY-jp).
;	;;MOVW      SP, #_stack /* Set stack pointer */ ; GNURL78
;	MOVW      SP, #LOWW(__STACK_ADDR_START) /* Set stack pointer */ ; CC-RL
#endif /* !defined(__CDT_PARSER__) */
}

/*
 * vRESTORE_CONTEXT_C
 * Restores the task Stack Pointer then use this to restore usCriticalNesting,
 * general purpose registers and the CS and ES registers of the selected task
 * from the task stack
 */
#pragma inline_asm vPortRESTORE_CONTEXT_C
static void vPortRESTORE_CONTEXT_C(void)
{
#if !defined(__CDT_PARSER__)
	MOVW      AX, !_pxCurrentTCB    ; Restore the Stack pointer.
	MOVW      HL, AX
	MOVW      AX, [HL]
	MOVW      SP, AX

;	It is assumed that usCriticalNesting, general purpose registers and
;	the CS and ES registers are to be restored by such as the following code
;	after this inline asm funcsion.
;	POP       AX                    ; Restore usCriticalNesting value.
;	MOVW      !_usCriticalNesting, AX
; or
;	MOVW      AX, [SP]
;	MOVW      !_usCriticalNesting, AX
;	POP       HL

;	POP       AX                    ; Restore the CS register.
;	MOV       CS, A
;	MOV       A, X                  ; Restore the ES register.
;	MOV       ES, A
;	POP       HL                    ; Restore general purpose registers.
;	POP       DE
;	POP       BC
;	POP       AX
#endif /* !defined(__CDT_PARSER__) */
}
